/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2013                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include <config.h>
#include "SmallObjAllocator.h"

#include "BlockAllocator.h"

using namespace std;
using namespace pearl;


//--- Private data structures -----------------------------------------------

struct SmallObjAllocator::tAllocator
{
    /// Underlying serial allocators
    alloc_container mSerial;
};


//---------------------------------------------------------------------------
//
//  class SmallObjAllocator (serial code)
//
//---------------------------------------------------------------------------

//--- Constructors & destructor ---------------------------------------------

SmallObjAllocator::SmallObjAllocator()
{
    // SmallObjAllocator is a singleton, i.e., the one and only instance
    // can initialize the specific allocator data
    assert(!mAllocator);
    mAllocator = new SmallObjAllocator::tAllocator;
}


SmallObjAllocator::~SmallObjAllocator()
{
  // Release allocators
  alloc_container::iterator it = mAllocator->mSerial.begin();
  while (it != mAllocator->mSerial.end())
  {
    delete it->second;
    ++it;
  }

  // Release allocator data
  delete mAllocator;
}


//--- Memory allocation routines --------------------------------------------

void* SmallObjAllocator::alloc(std::size_t objectSize) const throw(bad_alloc)
{
  // Retrieve memory for object
  return alloc(mAllocator->mSerial, objectSize);
}


void SmallObjAllocator::dealloc(void* deadObject, std::size_t objectSize) const
{
  // Release memory for object
  dealloc(mAllocator->mSerial, deadObject, objectSize);
}
