/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2013                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include <config.h>
#include "SmallObjAllocator.h"

#include <pearl/Error.h>

#include "BlockAllocator.h"

using namespace std;
using namespace pearl;


//--- Static member variables -----------------------------------------------

const SmallObjAllocator*       SmallObjAllocator::mInstance  = NULL;
SmallObjAllocator::tAllocator* SmallObjAllocator::mAllocator = NULL;


//---------------------------------------------------------------------------
//
//  class SmallObjAllocator (programming-model independent code)
//
//---------------------------------------------------------------------------

//--- Singleton interface ---------------------------------------------------

void SmallObjAllocator::registerAllocator(const SmallObjAllocator* allocator)
{
  assert(allocator);

  if (mInstance)
    throw FatalError("SmallObjAllocator::registerAllocator(const SmallObjAllocator*) -- "
                     "Allocator already registered.");

  mInstance = allocator;
}


void SmallObjAllocator::deregisterAllocator()
{
  assert(mInstance);

  delete mInstance;
  mInstance = NULL;
}


//--- Internal memory allocation routines (protected) -----------------------

void* SmallObjAllocator::alloc(alloc_container& container,
                               std::size_t      objectSize) const
                                                            throw(bad_alloc)
{
  // Search allocator for given size; create new one if necessary
  alloc_container::iterator it = container.lower_bound(objectSize);
  if (it == container.end() ||
      container.key_comp()(objectSize, it->first))
    it = container.insert(it,
                          alloc_container::value_type(objectSize,
                                                      new BlockAllocator(objectSize)));

  // Retrieve memory for object
  return it->second->alloc();
}


void SmallObjAllocator::dealloc(alloc_container& container,
                                void*            deadObject,
                                std::size_t      objectSize) const
{
  // Search allocator for given size
  alloc_container::iterator it = container.find(objectSize);

  // If allocator was not found, we are in trouble...
  assert(it != container.end());

  // Release memory for object
  it->second->dealloc(deadObject);
}
